<?php
/*
Plugin Name: iThemeland WooCommerce Dynamic Prices By User Role Plugin
Plugin URI: http://ithemelandco.com/Plugins/Woocommerce-Dynamic-Prices-By-User-Role/
Description: Prices by User Role for WooCommerce plugin facilitates Administrator to offer different prices for all different roles
Author: iThemeland
Version: 1.8
Tested up to: WP 6.0.1
Author URI: http://ithemelandco.com/
Text Domain: it-woocmmerce-price-rule
Domain Path: /languages/
 */
// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

define('plugin_dir_url_ithemeland_pricerule', plugin_dir_url(__FILE__));
define('_IT_PLUGIN_woocommerce_price_rule_textdomain_', 'it-woocmmerce-price-rule');

if ( !defined( 'plugin_dir_ithemeland_pricerule' ) ) {
    define( 'plugin_dir_ithemeland_pricerule', plugin_dir_path( __FILE__ ) );
}
if ( !defined( 'plugin_dir_path_ithemeland_price_by_user_rule' ) ) {
    define( 'plugin_dir_path_ithemeland_price_by_user_rule', plugin_dir_ithemeland_pricerule  );
}

class it_WC_Dynamic_Pricing
{
    //public $settgins;
    public $buffer_on = true;

    public function __construct()
    {
        if (is_admin()) {
            require 'admin/admin-init.php';
            add_action('admin_head', array($this, 'admin_enqueu'));
        }
        //For fetch user role
        add_action('init', array($this, 'init_user_info'), 10);


        add_action('pre_get_posts', array($this, 'hide_product_posts'));

        /******* For Cart *****/
        //For product simple in cart(non shop)
        add_filter('woocommerce_product_get_price', array($this, 'wc_product_get_price'), 20, 2);
        //For product variable in cart(non shop)
        add_filter('woocommerce_product_variation_get_price', array($this, 'wc_product_get_price'), 20, 2);
        // support to WooCommerce Product Bundles
        add_filter('woocommerce_bundle_get_base_price', array($this, 'get_price'), 5, 2);


        /******* For Shop *****/
        //For product simple and variable in shop(non cart)
        add_filter('woocommerce_get_price_html', array($this, 'wc_get_price_html'), 11, 2);
        //For product simple and variable in shop(non cart)
        add_filter('woocommerce_get_variation_price_html', array($this, 'wc_get_price_html'), 11, 2);

        add_filter('woocommerce_product_is_on_sale', array($this, 'product_is_on_sale'), 10, 2);


        //Hide add To cart for shop page
        add_filter('woocommerce_loop_add_to_cart_link', array($this, 'hide_add_to_cart_loop'), 10, 2);
        //Hide Add to cart for single page
        add_action('woocommerce_before_booking_form', array($this, 'on_before_booking_form'), 1);
        add_action('woocommerce_before_add_to_cart_button', array(&$this, 'on_before_add_to_cart_button'), 1);
        add_action('woocommerce_after_add_to_cart_button', array(&$this, 'on_after_add_to_cart_button'), 999);
        //Remove add to cart with jqury for trust
        add_action('woocommerce_single_product_summary', array($this, 'remove_add_to_cart_with_jqury'), 10);

        //     add_filter('woocommerce_variation_prices_price', array($this, 'on_get_variation_prices_price'), 10, 3);

        //if can't add to cart so can't purchase
        add_filter('woocommerce_is_purchasable', array($this, 'is_purchasable'), 15, 2);
        add_filter('woocommerce_variation_is_purchasable', array($this, 'is_purchasable'), 15, 2);
    }

    public function on_get_variation_prices_price($price, $variation)
    {
        return $this->get_discounted_price($price, $variation);

    }

    public function get_discounted_price($price, $product)
    {
        $id = $product->get_id();
        $discount_price = $price;
        if (is_admin() && !(defined('DOING_AJAX') && DOING_AJAX)) {
            return $price;
        }

        if ($product->is_type('variation')) {
            if ('yes' == (get_post_meta($id, 'it_product_disable_discount_variation', true))) {
                return $price;
            }
        } else {
            if ('yes' == (get_post_meta($id, 'it_product_disable_discount', true))) {
                return $price;
            }
        }

        if (empty($product) || empty($price)) {
            return $price;
        }
        $current_user_roles = $this->user_role['role'];

        if (isset($this->user_role['general']['roles'][$current_user_roles]['on']) &&
            $this->user_role['general']['roles'][$current_user_roles]['amount'] != ''
        ) {
            $for_apply = $this->user_role['general']['roles'][$current_user_roles]['for_apply'];
            $amount = $this->user_role['general']['roles'][$current_user_roles]['amount'];
            $type = $this->user_role['general']['roles'][$current_user_roles]['type'];

            if ($for_apply == 'on_sale') {
                $discount_price = $product->get_sale_price();
                if ($discount_price > 0) {
                } else {
                    $discount_price = $product->get_regular_price();
                    return $discount_price;
                }
            } elseif ($for_apply == 'role') {

                if ($product->is_type('variation')) {
                    $pricing_rules_variation = get_post_meta($id, 'pricing_rules_variation', true);
                    $discount_price = @$pricing_rules_variation['price_rule'][$current_user_roles]['price'];
                    if ($discount_price != 0 || $discount_price > 0 || $discount_price != '') {
                    } else {
                        $discount_price = $product->get_regular_price();
                        return $discount_price;
                    }
                } else {
                    $pricing_rules_product = get_post_meta($id, 'pricing_rules_product', true);
                    $discount_price = @$pricing_rules_product['price_rule'][$current_user_roles]['amount'];
                    if ($discount_price != 0 || $discount_price > 0 || $discount_price != '') {
                    } else {
                        $discount_price = $product->get_regular_price();
                        return $discount_price;
                    }
                }

            } else {
                $discount_price = $product->get_regular_price();
            }

            if (is_null($discount_price) || $discount_price == '' || $discount_price <= 0) {
                return $price;
            }

            $working_price = $this->discunt_type_calculator($discount_price, $amount, $type);

            $discount_price = $this->do_markup_discunt($discount_price, $working_price, $current_user_roles);


            if ($discount_price <= 0) {
                $discount_price = __('free', 'woocommerce');
            }
        } else {

            if ($product->is_type('variation')) {
                $pricing_rules_variation = get_post_meta($id, 'pricing_rules_variation', true);
                $price_role = @$pricing_rules_variation['price_rule'][$current_user_roles]['price'];
                if ($price_role != 0 || $price_role > 0 || $price_role != '') {
                    $discount_price = $price_role;
                }
            } else {
                $pricing_rules_product = get_post_meta($id, 'pricing_rules_product', true);
//                print_r($pricing_rules_product);
//                die;
                $price_role = @$pricing_rules_product['price_rule'][$current_user_roles]['amount'];
                if ($price_role != 0 || $price_role > 0 || $price_role != '') {
                    $discount_price = $price_role;
                }
            }

        }
        return $discount_price;
    }

    public function admin_enqueu()
    {
        wp_enqueue_style('it-public', $this->plugin_url() . '/assets/admin/public.css');
        wp_enqueue_script('it-dependsOn', $this->plugin_url() . '/assets/admin/dependsOn-1.0.1.min.js', array('jquery'));
    }

    public static function plugin_url()
    {
        return plugins_url(basename(plugin_dir_path(__FILE__)), basename(__FILE__));
    }

    public static function plugin_path()
    {
        return untrailingslashit(plugin_dir_path(__FILE__));
    }

    //Get User Info
    public function init_user_info()
    {
        $this->user_role['general'] = get_option('_s_pricing_discount_rules_hide');
        $this->user_role['general']['roles'] = get_option('_s_pricing_discount_rules');

        if (!is_user_logged_in()) {
            $this->user_role['role'] = 'unregister';
            $this->user_role['add_to_cart_text'] = '<span class="it-hide-cart">' . $this->add_to_cart_text() . '</span>';
            // $this->user_role['price_empty'] = '<span class="it-addtocart-btn">' . $this->hide_empty_text() . '</span>';
            $this->user_role['suffix_price_text'] = '<span class="it-price-suffix">' . $this->suffix_price_text('unregister') . '</span>';
            $this->user_role['price_text'] = '<span class="it-hide-price">' . $this->price_text() . '</span>';
        } else {
            $user_id = get_current_user_id();
            $user = get_user_by('id', $user_id);
            $current_user_rule = $this->get_currnet_user_role();
            $user_role = $current_user_rule[0];
            $this->user_role['role'] = $user_role;
            $this->user_role['add_to_cart_text'] = '<span class="it-hide-cart">' . $this->add_to_cart_text($user_role) . '</span>';
            //$this->user_role['price_empty'] = '<span class="it-addtocart-btn">' . $this->hide_empty_text($user_role) . '</span>';
            $this->user_role['suffix_price_text'] = '<span class="it-price-suffix">' . $this->suffix_price_text($user_role) . '</span>';
            $this->user_role['price_text'] = '<span class="it-hide-price">' . $this->price_text($user_role) . '</span>';

        }
    }

    //For add to cart in single page
    public function hide_product_posts($query)
    {
        $idproducts = array();
        $option_user_role = get_option('it_pricing_product_hide_user_role');
        if (is_array($option_user_role) && count($option_user_role) > 0) {
            foreach ($option_user_role as $key => $id) {
                $idproducts = $option_user_role[$this->user_role['role']];
            }
            $query->set('post__not_in', $idproducts);
        }
    }

    //For add to cart in single page
    public function on_before_booking_form()
    {
        remove_action('woocommerce_before_add_to_cart_button', array(&$this, 'on_before_add_to_cart_button'), 1);
        global $product, $it_wc_rule;
        if (!$this->check_can_add_to_cart()) {
            $this->buffer_on = ob_start();
        }
    }

    //Hide add to cart single page
    public function on_before_add_to_cart_button()
    {
        global $it_wc_rule, $product;

        if (!$this->check_can_add_to_cart()) {
            $this->buffer_on = ob_start();
        }
    }

    //Hide add to cart single page
    public function on_after_add_to_cart_button()
    {
        global $product;
		$product_id = $product->get_id();
        if (!$this->check_can_add_to_cart(absint($product_id))) {
            if ($this->buffer_on) {
                ob_end_clean();
            }
            $html = do_shortcode(wpautop(wptexturize(stripslashes($this->user_role['general']['add_to_cart'][$this->user_role['role']]['text']))));

            // Variable product price handling
            if ($product->is_type('variable')) {
                ?>
                <div class="single_variation_wrap">
                    <div class="single_variation woocommerce-variation"></div>
                    <div class="variations_button">
                        <?php echo $html; ?>
                        <input type="hidden" name="product_id" value="<?php echo absint($product->id); ?>"/>
                        <input type="hidden" name="variation_id" class="variation_id" value="0"/>
                    </div>
                </div>
                <?php
            } else {
                echo $html;
            }
        }
    }

    //can purchase
    public function is_purchasable($purchasable, $product)
    {
        //check single product For Guest
        if (!$this->check_can_add_to_cart()) {
            return false;
        }
        return $purchasable;
    }

    //add to cart text
    public function add_to_cart_text($user_role = 'unregister')
    {
        $text = '';
        if (isset($this->user_role['general']['add_to_cart'][$user_role]['on'])) {
            $text = stripslashes($this->user_role['general']['add_to_cart'][$user_role]['text']);
        }
        return $text;
    }

    //add to cart text
    public function hide_empty_text($user_role = 'unregister')
    {
        $text = '';
        if (isset($this->user_role['general']['price_empty'][$user_role]['on'])) {
            $text = stripslashes($this->user_role['general']['price_empty'][$user_role]['text']);
        }
        return $text;
    }

    //price text
    public function price_text($user_role = 'unregister')
    {
        $text = '';
        if (isset($this->user_role['general']['price'][$user_role]['on'])) {
            $text = stripslashes($this->user_role['general']['price'][$user_role]['text']);
        }

        return $text;
    }

    //hide add to cart shop page
    public function hide_add_to_cart_loop($link, $product)
    {
        //check single product For Guest
        global $product;
        $product_id = $product->get_id();

        if (!$this->check_can_add_to_cart($product_id)) {

            return $this->user_role['add_to_cart_text'];
        }
//        if (!$this->user_role['show_add_to_cart'] && $product->product_type != 'variable') {
//            return '';
//        }

        return $link;
    }

    //If add to cart don't remove
    public function remove_add_to_cart_with_jqury()
    {
        global $product;
        if (!$this->check_can_add_to_cart()) {

            if (isset($product) && $product->product_type == 'variable') {
                $hide_quantity = "$('.single_variation_wrap .variations_button .quantity' ).hide();";
                $inline_js
                    = "
                        $( '.single_variation_wrap .variations_button button' ).hide();" .
                    $hide_quantity .
                    "$( document).on( 'woocommerce_variation_has_changed', function() {
                         $( '.single_variation_wrap .variations_button button' ).hide();"
                    . $hide_quantity .
                    "});";
                wc_enqueue_js($inline_js);
            } else {
                $inline_js = "$( '.cart button.single_add_to_cart_button' ).hide();";
                wc_enqueue_js($inline_js);
            }
        }
    }

    //For Cart Discunt
    public function wc_product_get_price($price, $product)
    {
        $discount_price = $price;
        if (is_admin() && !(defined('DOING_AJAX') && DOING_AJAX)) {
            return $price;
        }

        if ($product->is_type('variation')) {
            if ('yes' == (get_post_meta($product->get_id(), 'it_product_disable_discount_variation', true))) {
                return $price;
            }
        } else {
            if ('yes' == (get_post_meta($product->get_id(), 'it_product_disable_discount', true))) {
                return $price;
            }
        }
//
        //        foreach ( $variation_ids as $variation_id ) {
        //			if ('yes' == (get_post_meta($variation_id, 'it_product_disable_discount_variation', true))) {
        //				//echo $price;

        //				return apply_filters('it_get_price_html', $price, $product);
        //			}
        //		}
        //	}
        //	else
        //	{
        //


        if (empty($product) || empty($price)) {
            return $price;
        }
        $current_user_roles = $this->user_role['role'];
		if (!is_user_logged_in()) {
            $current_user_roles = 'customer';
		}
		
        if (isset($this->user_role['general']['roles'][$current_user_roles]['on']) &&
            $this->user_role['general']['roles'][$current_user_roles]['amount'] != ''
        ) {
			
            $for_apply = $this->user_role['general']['roles'][$current_user_roles]['for_apply'];
            $amount = $this->user_role['general']['roles'][$current_user_roles]['amount'];
            $type = $this->user_role['general']['roles'][$current_user_roles]['type'];

            if ($for_apply == 'on_sale') {
                $discount_price = $product->get_sale_price();
                if ($discount_price > 0) {
                } else {
                    $discount_price = $product->get_regular_price();
                    return $discount_price;
                }
            } elseif ($for_apply == 'role') {

                if ($product->is_type('variation')) {
                    $pricing_rules_variation = get_post_meta($product->get_id(), 'pricing_rules_variation', true);
                    $discount_price = @$pricing_rules_variation['price_rule'][$current_user_roles]['price'];
                    if ($discount_price != 0 || $discount_price > 0 || $discount_price != '') {
                    } else {
                        $discount_price = $product->get_regular_price();
                        return $discount_price;
                    }
                } else {
                    $pricing_rules_product = get_post_meta($product->get_id(), 'pricing_rules_product', true);
                    $discount_price = @$pricing_rules_product['price_rule'][$current_user_roles]['amount'];
                    if ($discount_price != 0 || $discount_price > 0 || $discount_price != '') {
                    } else {
                        $discount_price = $product->get_regular_price();
                        return $discount_price;
                    }
                }

            } else {

                $discount_price = $product->get_regular_price();
            }

            if (is_null($discount_price) || $discount_price == '' || $discount_price <= 0) {
                return $price;
            }

            $working_price = $this->discunt_type_calculator($discount_price, $amount, $type);

            $discount_price = $this->do_markup_discunt($discount_price, $working_price, $current_user_roles);

            if ($discount_price <= 0) {
               // $discount_price = __('free', 'woocommerce');
            }
        } else {
			
			$price_role=0;
            if ($product->is_type('variation')) {
                $pricing_rules_variation = get_post_meta($product->get_id(), 'pricing_rules_variation', true);
				if(isset($pricing_rules_variation['price_rule']))
				{
					$price_role = $pricing_rules_variation['price_rule'][$current_user_roles]['price'];	
				}
                
				if(isset($pricing_rules_product['price_rule'])){
					$price_role = $pricing_rules_product['price_rule'][$current_user_roles]['amount'];	
				}				
                if ($price_role != 0 || $price_role > 0 || $price_role != '') {
                    $discount_price = $price_role;
                }
            } else {
                $pricing_rules_product = get_post_meta($product->get_id(), 'pricing_rules_product', true);
				if(is_array($pricing_rules_product)){
					if(isset($pricing_rules_product['price_rule'])){
						$price_role = $pricing_rules_product['price_rule'][$current_user_roles]['amount'];	
					}
					
					if ($price_role != 0 || $price_role > 0 || $price_role != '') {
						$discount_price = $price_role;
					}
				}
            }

        }
        return $discount_price;
    }

    //add to cart text
    public function do_markup_discunt($discount_price, $working_price, $current_user_roles)
    {
		$price=0;
        if ($this->user_role['general']['roles'][$current_user_roles]['type_calc'] == 'discount') {
            $price = $discount_price - $working_price;
        } elseif ($this->user_role['general']['roles'][$current_user_roles]['type_calc'] == 'markup') {
            $price = $discount_price + $working_price;
        }

        return ($price > 0 ) ? $price : 0;
    }

    //add to cart text
    public function suffix_price_text($user_role = 'unregister')
    {
        $text = '';
        if (@$this->user_role['general']['roles']['suffix']['option'] == 'general') {
            $text = $this->user_role['general']['roles']['suffix']['suffix_text'];
        } elseif (@$this->user_role['general']['roles']['suffix']['option'] == 'role') {

            if (isset($this->user_role['general']['roles']['suffix']['role'][$user_role]['text'])) {
                $text = $this->user_role['general']['roles']['suffix']['role'][$user_role]['text'];
            }
        }
        return $text;
    }

    //Calculator discount
    public function discunt_type_calculator($price_base, $amount, $type)
    {
        $price_base = $price_base < 0 ? 0 : $price_base;
        //echo $type['type'];
        //die('a');
        switch ($type) {
            case 'percent_product':
                $discount = $price_base * ($amount / 100);
                break;

            case 'fixed_product':
                $discount = $amount;
                break;
        }
        $discount = ceil($discount * pow(10, get_option('woocommerce_price_num_decimals')) - 0.5) * pow(10, -((int)get_option('woocommerce_price_num_decimals')));
        if ($discount < 0)
            return 0;
        else
            return $discount;
    }

    //Get Current user role
    public function get_currnet_user_role()
    {
        $current_user = wp_get_current_user();
        if ($current_user)
            return $current_user->roles;
        else
            return false;
    }

    public function get_base_product_id_it($product)
    {

        return $product instanceof WC_Data && $product->is_type('variation') ?
            $product->get_parent_id() :
            $product->get_id();
    }

    //For Shop Discunt
    public function wc_get_price_html($price, $product)
    {
        $product_type = $product->get_type();
        $product_id = $this->get_base_product_id_it($product);

        if ($product_type == 'variable') {

            if (!$this->check_can_see_price($product_id)) {
                return apply_filters('it_get_price_html', $this->user_role['price_text'], $product);
            }
			$price_temp = $this->get_variable_price_html($price, $product);
			//if($price != $price_temp)
			//{
			//	$price='<del>'.$price .'</del> <ins>'.$price_temp.'</ins>';
			//}
            return apply_filters('it_get_price_html', $price_temp, $product);
			
        }

        if ('yes' == (get_post_meta($product_id, 'it_product_disable_discount', true))) {
            return apply_filters('it_get_price_html', $price, $product);
        }

        if ((is_admin() && !(defined('DOING_AJAX') && DOING_AJAX))) {

        } else {

            if (!$this->check_can_see_price($product_id)) {

                return apply_filters('it_get_price_html', $this->user_role['price_text'], $product);
            }

            $product_type = $product->get_type();

            switch ($product_type) {
                case 'simple':
                case 'variation':
                    $price = $this->get_simple_price_html($price, $product, $product->get_id());
                    break;
                case 'variable':
                    $price = $this->get_variable_price_html($price, $product);
                    break;
            }
        }
        return apply_filters('it_get_price_html', $price, $product);
    }

    //for variable product
    public function get_variable_price_html($price, $product)
    {
        $variation_prices = $product->get_variation_prices(true);
        $variation_regular_prices = $variation_prices['regular_price'];
        $variation_sale_prices = $variation_prices['sale_price'];

        $current_user_roles = $this->user_role['role'];
        $for_apply = $this->user_role['general']['roles'][$current_user_roles]['for_apply'];

        $variation_new_prices = $this->get_variation_new_prices($price, $product);

		$regular_price_html=$price;
        // $variation_new_prices = $product->get_variation_prices(true);


        if (!empty($variation_new_prices) && isset($variation_new_prices['discount_price'])) {
			
			//$new_prices['base_price'][$variation_id] = $price;
			//$new_prices['discount_price'][$variation_id] = $discount_price;
            $min_price_base = current($variation_new_prices['base_price']);

            $min_price = current($variation_new_prices['discount_price']);
            $max_price = end($variation_new_prices['discount_price']);
			
            $your_price_html_base = $min_price_base;
			
            $your_price_html = $min_price !== $max_price ? sprintf(_x('%1$s&ndash;%2$s', 'Price range: from-to', 'woocommerce'), wc_price($min_price), wc_price($max_price)) : wc_price($min_price);
			
			$regular_price_html='<del>'.$your_price_html_base .'</del> <ins>'.$your_price_html.'</ins>';			
			//$regular_price_html=$your_price_html;
        }

        if (isset($this->user_role['general']['roles'][$current_user_roles]['on'])) {

            if ($for_apply == 'on_sale' && $product->is_on_sale()) {
                $min_price = current($variation_sale_prices);
                $max_price = end($variation_sale_prices);

                if ($min_price == 0 && $max_price == 0) {
                    $sale_price_html = __('Free!', 'woocommerce');
                } else {
                    $sale_price_html = $min_price !== $max_price ? sprintf(_x('%1$s&ndash;%2$s', 'Price range: from-to', 'woocommerce'), wc_price($min_price), wc_price($max_price)) : wc_price($min_price);
                    if (empty($your_price_html)) {
                        $sale_price_html = sprintf('<span class="%s">%s</span>', 'sale_price', $sale_price_html);
                    } else {
                        $sale_price_html = sprintf('<span class="%s"><del>%s</del></span>', 'sale_price', $sale_price_html);
                    }

                }

            } else {

                $min_price = current($variation_regular_prices);
                $max_price = end($variation_regular_prices);
				
				$min_price_base = current($variation_new_prices['base_price']);

				$min_price = current($variation_new_prices['discount_price']);
				$max_price = end($variation_new_prices['discount_price']);
			
                if ($min_price == 0 && $max_price == 0) {
                    $regular_price_html = __('Free!', 'woocommerce');
                } else {
                    $regular_price_html = $min_price !== $max_price ? sprintf(_x('%1$s&ndash;%2$s', 'Price range: from-to', 'woocommerce'), wc_price($min_price), wc_price($max_price)) : wc_price($min_price);
                    if (empty($your_price_html)) {
                        $regular_price_html = sprintf('<span class="%s">%s</span>', 'regular_price', $regular_price_html);
                    } else {
                        $regular_price_html = '<span class="sale_price"><del>'.$min_price_base.'</del><ins>'.wc_price($min_price).'</ins></span>'. $this->user_role['suffix_price_text'];
                    }

                }
            }

        } else {
            return $regular_price_html . $this->user_role['suffix_price_text'];
        }
        return $regular_price_html;
        //return $regular_price_html . $sale_price_html . $your_price_html.$this->user_role['suffix_price_text'];
        //    return $price;

//        if (!empty($regular_price_html)) {
//
//            if (empty($sale_price_html) && empty($your_price_html)) {
//                $regular_price_html = sprintf('<span class="%s">%s %s</span>', 'ywcrbp_regular_price', $regular_price_txt, $regular_price_html);
//            } else {
//                $regular_price_html = sprintf('<span class="%s"><del>%s %s</del></span>', 'ywcrbp_regular_price', $regular_price_txt, $regular_price_html);
//            }
//        }
//
//        if (!empty($sale_price_html)) {
//            if (empty($your_price_html)) {
//                $sale_price_html = sprintf('<span class="%s">%s %s</span>', 'ywcrbp_sale_price', $sale_price_txt, $sale_price_html);
//            } else {
//                $sale_price_html = sprintf('<span class="%s"><del>%s %s</del></span>', 'ywcrbp_sale_price', $sale_price_txt, $sale_price_html);
//            }
//        }
//        if (!empty($your_price_html)) {
//
//            $your_price_html = sprintf('<span class="%s">%s %s</span>', 'ywcrbp_your_price', $your_price_txt, $your_price_html);
//        }


    }

    //get new price for varialbe
    public function get_variation_new_prices($price, $product)
    {
        $new_prices = array();
		//$discount_price='no_price';
        $variation_ids = version_compare(WC()->version, '2.7.0', '>=') ? $product->get_visible_children() : $product->get_children(true);

        foreach ($variation_ids as $variation_id) {

            if ('yes' == (get_post_meta($variation_id, 'it_product_disable_discount_variation', true))) {
                return $price;
            }
            $variation = wc_get_product($variation_id);
            if ($variation instanceof WC_Product_Variation) {

                $current_user_roles = $this->user_role['role'];
                if (isset($this->user_role['general']['roles'][$current_user_roles]['on']) && !empty($this->user_role['general']['roles'][$current_user_roles]['amount'])) {

                    $for_apply = $this->user_role['general']['roles'][$current_user_roles]['for_apply'];
                    $amount = $this->user_role['general']['roles'][$current_user_roles]['amount'];
                    $type = $this->user_role['general']['roles'][$current_user_roles]['type'];
                    if ($for_apply == 'role') {
                        $pricing_rules_variation = (get_post_meta($variation_id, 'pricing_rules_variation', true));
                        $discount_price = @$pricing_rules_variation['price_rule'][$current_user_roles]['price'];
                        if ($discount_price != 0 || $discount_price > 0 || $discount_price != '') {
                        } else {
                            $discount_price = $variation->get_regular_price();
//                            $price_sale = $variation->get_sale_price();
//                            if ($discount_price <= 0) {
//                                return __('Free', 'woocommerce');
//                            }
                            //  return $show . $this->user_role['suffix_price_text'];
                        }

//                        print_r($pricing_rules_variation);
//                        echo $discount_price;
//                        $discount_price=32;
                    } elseif ($for_apply == 'on_sale') {
                        $discount_price = $variation->get_sale_price();
                        if ($discount_price > 0) {
                        } else {
                            $discount_price = $variation->get_regular_price();
                        }
                    } else {
                        $discount_price = $variation->get_regular_price();
                    }
//                     print_r($variation->get_sale_price());
//                     echo  'e';
                    //print_r($variation);
                    if (is_null($discount_price) || $discount_price == '' || $discount_price <= 0) {
                        return $price;
                    }

                    // echo $discount_price;
                    $working_price = $this->discunt_type_calculator($discount_price, $amount, $type);


                    $discount_price = $this->do_markup_discunt($discount_price, $working_price, $current_user_roles);


                    if ($discount_price <= 0) {
                        $discount_price = __('free', 'woocommerce');
                    }
                } 
				else {
					
                    $pricing_rules_variation = get_post_meta($variation_id, 'pricing_rules_variation', true);
					if(is_array($pricing_rules_variation) && isset($pricing_rules_variation['price_rule'][$current_user_roles])){
						$price_role = $pricing_rules_variation['price_rule'][$current_user_roles]['price'];
						
						if ($price_role != 0 || $price_role > 0 || $price_role != '') {
							$discount_price = $price_role;
						}
					}
                }
                // $new_price = $this->get_role_based_price($variation);

                if (isset($discount_price) && 'no_price' !== $discount_price) {
                    $new_prices['base_price'][$variation_id] = $price;
                    $new_prices['discount_price'][$variation_id] = $discount_price;
                }
            }
            asort($new_prices);
        }
        return $new_prices;
    }

    //Products simple for shop
    public function get_simple_price_html($price, $product, $product_id)
    {

        $discount_price = $price;
        $current_user_roles = $this->user_role['role'];
		if (!is_user_logged_in()) {
            $current_user_roles = 'customer';
		}
        if (isset($this->user_role['general']['roles'][$current_user_roles]['on']) &&
            $this->user_role['general']['roles'][$current_user_roles]['amount'] != ''
        ) {
			
            $for_apply = $this->user_role['general']['roles'][$current_user_roles]['for_apply'];
            $amount = $this->user_role['general']['roles'][$current_user_roles]['amount'];
            $type = $this->user_role['general']['roles'][$current_user_roles]['type'];
            if ($for_apply == 'on_sale') {
                $discount_price = $product->get_sale_price();
                if ($discount_price > 0) {
                } else {
                    $discount_price = $product->get_regular_price();
                    return wc_price($discount_price) . $this->user_role['suffix_price_text'];
                }
                $show = '<del><span class="amount">' . wc_price($discount_price) . '</span></del>';
            } elseif ($for_apply == 'role') {
                $pricing_rules_product = get_post_meta($product_id, 'pricing_rules_product', true);
                $discount_price = @$pricing_rules_product['price_rule'][$current_user_roles]['amount'];
                $show = '<del><span class="amount">' . wc_price($discount_price) . '</span></del>';
                if ($discount_price != 0 || $discount_price > 0 || $discount_price != '') {
                } else {
                    $discount_price = $product->get_regular_price();
                    $price_sale = $product->get_sale_price();
                    if ($price_sale != '') {
                        $show = '<del><span class="amount">' . wc_price($discount_price) . '</span></del>';
                        $show .= '<ins><span class="amount">' . wc_price($price_sale) . '</span></ins>';
                    } else {
                        $show = '<span class="amount">' . wc_price($discount_price) . '</span>';
                    }

                    if ($discount_price <= 0) {
                        return __('Free', 'woocommerce');
                    }
                    return $show . $this->user_role['suffix_price_text'];
                }

            } else {
				
                $show = '<del><span class="amount">' . wc_price($product->get_regular_price()) . '</span></del>';
                $discount_price = $product->get_regular_price();
                if ($discount_price <= 0) {
                    return __('Free', 'woocommerce');
                }
            }

            if (is_null($discount_price) || $discount_price == '' || $discount_price <= 0) {
                return $price . $this->user_role['suffix_price_text'];
            }

            $working_price = $this->discunt_type_calculator($discount_price, $amount, $type);
            $discount_price_do_markup_discunt = $this->do_markup_discunt($discount_price, $working_price, $current_user_roles);

            if ($discount_price_do_markup_discunt <= 0) {
                return __('Free', 'woocommerce');
            }
            if ($discount_price < $discount_price_do_markup_discunt) {
                $show = '<ins><span class="amount">' . wc_price($discount_price_do_markup_discunt) . '</span></ins>';
            } else {
                $show .= '<ins><span class="amount">' . wc_price($discount_price_do_markup_discunt) . '</span></ins>';
            }
//            echo '<pre>';
//            print_r($this->user_role);
            return $show . $this->user_role['suffix_price_text'];
        } else {
			$get_sale_price = $product->get_sale_price();
			if ($discount_price > 0) {
			} else {
				$get_sale_price = $product->get_regular_price();			
			}
            $price_role = 0;
            $pricing_rules_product = get_post_meta($product_id, 'pricing_rules_product', true);
			if(isset($pricing_rules_product['price_rule'])){
				$price_role = $pricing_rules_product['price_rule'][$current_user_roles]['amount'];	
			}			

			//echo $get_sale_price;
			//$get_regular_price = $product->get_regular_price();
            if ($price_role != 0 || $price_role > 0 || $price_role != '') {				
				$price = '<del><span class="amount">' . wc_price($get_sale_price) . '</span></del>';
				$price .= '<ins><span class="amount">' . wc_price($price_role) . '</span></ins>';  				
            }
        }
		
        return $price . $this->user_role['suffix_price_text'];
    }

    public function product_is_on_sale($is_on_sale, $product)
    {

        remove_filter('woocommerce_product_get_price', array($this, 'wc_product_get_price'), 5);
        remove_filter('woocommerce_product_is_on_sale', array($this, 'product_is_on_sale'), 10);
        $is_on_sale = $product->is_on_sale();
        add_filter('woocommerce_product_is_on_sale', array($this, 'product_is_on_sale'), 10, 2);
        add_filter('woocommerce_product_get_price', array($this, 'wc_product_get_price'), 5, 2);
        return $is_on_sale;
    }

    //check Hide price single page
    private function check_can_see_price($product_id = -1)
    {
//        global $product;

        //check single product For Guest
        if ($product_id > 0) {
            if ('yes' == (get_post_meta($product_id, 'it_product_hide_price_unregistered', true)) && $this->user_role['role'] == 'unregister') {
                return false;
            }

            //check single product for register user
            $hide_price_role = get_post_meta($product_id, 'it_pricing_product_price_user_role', false);
            $remove_product_price_roles = empty($hide_price_role) ? array() : current($hide_price_role);
            if (is_array($remove_product_price_roles) && in_array($this->user_role['role'], $remove_product_price_roles)) {
                return false;
            }
        }
        //Check general for all user
        if (isset($this->user_role['general']['price'][$this->user_role['role']]['on'])) {
            return false;
        }
        return true;
    }

    //check Hide add to cart single page
    public function check_can_add_to_cart($product_id = -1)
    {
        global $product;


        if ($product_id > 0) {

            //check single product For Guest
            if ('yes' == (get_post_meta($product_id, 'it_product_hide_price_unregistered', true)) && $this->user_role['role'] == 'unregister') {
                return false;
            }

            //check single product for register user
            $hide_add_to_cart = get_post_meta($product_id, 'it_pricing_product_add_to_cart_user_role', false);
            $remove_product_add_ro_cart_roles = empty($hide_add_to_cart) ? array() : current($hide_add_to_cart);
            if (is_array($remove_product_add_ro_cart_roles) && in_array($this->user_role['role'], $remove_product_add_ro_cart_roles)) {
                return false;
            }
        }
        //Check general for all user
        if (isset($this->user_role['general']['add_to_cart'][$this->user_role['role']]['on'])) {
            return false;
        }
        return true;
    }

}

new it_WC_Dynamic_Pricing();


class ithemeland_class_fetch_admin
{

    public function __construct()
    {
        add_filter('woocommerce_product_data_tabs', array($this, 'add_product_tab'));
        add_action('woocommerce_product_data_panels', array($this, 'add_price_adjustment_data_fields'));
        add_action('woocommerce_product_options_general_product_data', array($this, 'show_product_price_rule'));
        add_action('woocommerce_process_product_meta', array($this, 'woo_add_custom_general_fields_save'));
        add_action('woocommerce_save_product_variation', array($this, 'save_product_variation_meta'), 25, 2);
        add_action('woocommerce_variation_options_pricing', array($this, 'show_product_variation_price_rule'), 10, 3);
    }

    public function add_product_tab($product_data_tabs)
    {
        global $product;
        $product_data_tabs['product_price_descount'] = array(
            'label' => __('Hide Roles Price', _IT_PLUGIN_woocommerce_price_rule_textdomain_),
            'target' => 'product_price_discount_data',
            'id' => 'it_pricing_product_role_based_settings',
            'class' => Array('it_pricing_settings'),
        );
        return $product_data_tabs;
    }

    public function woo_add_custom_general_fields_save($post_id)
    {
        //to update product price for unregistered users
        $woocommerce_adjustment_field = (isset($_POST['it_product_disable_discount']) && ($_POST['it_product_disable_discount'] == 'on')) ? 'yes' : 'no';
        if (!empty($woocommerce_adjustment_field)) {
            update_post_meta($post_id, 'it_product_disable_discount', $woocommerce_adjustment_field);
        }
        // pricing_rules_product[price_rule][$role_id][amount]
        //to update product price for unregistered users
        $pricing_rules_product = $_POST['pricing_rules_product'];
        if (!empty($pricing_rules_product)) {
            update_post_meta($post_id, 'pricing_rules_product', $pricing_rules_product);
        }

        //to update product price for unregistered users
        $woocommerce_adjustment_field = (isset($_POST['it_product_hide_price_unregistered']) && ($_POST['it_product_hide_price_unregistered'] == 'on')) ? 'yes' : 'no';
        if (!empty($woocommerce_adjustment_field)) {
            update_post_meta($post_id, 'it_product_hide_price_unregistered', $woocommerce_adjustment_field);
        }

        //to update product role based hide price
        $woocommerce_adjustment_field = $_POST['it_pricing_product_price_user_role'];
        if (!empty($_POST['it_pricing_product_price_user_role'])) {
            update_post_meta($post_id, 'it_pricing_product_price_user_role', $woocommerce_adjustment_field);
        } else {
            delete_post_meta($post_id, 'it_pricing_product_price_user_role');
        }

        //to update product role based hide price
        $woocommerce_adjustment_field = $_POST['it_pricing_product_add_to_cart_user_role'];
        if (!empty($woocommerce_adjustment_field)) {
            update_post_meta($post_id, 'it_pricing_product_add_to_cart_user_role', $woocommerce_adjustment_field);
        } else {
            delete_post_meta($post_id, 'it_pricing_product_add_to_cart_user_role');
        }

        // For Hide Products
        $woocommerce_adjustment_field = $_POST['it_pricing_product_hide_user_role'];

        if (!empty($woocommerce_adjustment_field)) {
            update_post_meta($post_id, 'it_pricing_product_hide_user_role', $woocommerce_adjustment_field);
        } else {
            delete_post_meta($post_id, 'it_pricing_product_hide_user_role');
        }

        if (is_array($woocommerce_adjustment_field)) {
            foreach ($woocommerce_adjustment_field as $key => $id) {
                $arrray_id[$id] = array($post_id);
            }
        }

        $option_user_role = get_option('it_pricing_product_hide_user_role');
        foreach ($option_user_role as $key => $id) {
            if (in_array($post_id, $id)) {
                unset($option_user_role[$key]);
            }
        }

        if (is_array($option_user_role) && count($option_user_role) > 0) {
            $array_merge_save = array_merge_recursive($option_user_role, $arrray_id);
            update_option('it_pricing_product_hide_user_role', $array_merge_save);
        } else {
            update_option('it_pricing_product_hide_user_role', $arrray_id);
        }
    }

    public function add_price_adjustment_data_fields()
    {
        global $woocommerce, $post;
        ?>
        <!-- id below must match target registered in above add_my_custom_product_data_tab function -->
        <div id="product_price_discount_data" class="panel woocommerce_options_panel hidden">
            <?php include('admin/woocommerce_product_single.php'); ?>
        </div>
        <?php
    }

    public function show_product_price_rule()
    {

        wc_get_template('admin/woocommerce_product_single-general.php', array(), '', plugin_dir_path_ithemeland_price_by_user_rule);
    }

    public function show_product_variation_price_rule($loop, $variation_data, $variation)
    {
        wc_get_template('admin/woocommerce_product_single-variation.php', array('loop' => $loop, 'variation_data' => $variation_data, 'variation' => $variation), '', plugin_dir_path_ithemeland_price_by_user_rule);

    }

    public function save_product_variation_meta($variation_id, $loop)
    {

        $variation_rule = isset($_REQUEST['it_product_disable_discount_variation'][$loop]['disable_discount']) ? 'yes' : 'no';

        update_post_meta($variation_id, 'it_product_disable_discount_variation', $variation_rule);

//        $pricing_rules_product = $_POST['pricing_rules_product'];
//        if (!empty($pricing_rules_product)) {
        update_post_meta($variation_id, 'pricing_rules_variation', $_REQUEST['pricing_rules_variation'][$loop]);
//        }

    }

}

new ithemeland_class_fetch_admin();
?>